package cn.ischoolcode.javabook.dushi.chat;

/* 
 * 爱校码
 * http://www.ischoolcode.cn/
 *  
 */

import java.util.*;
import java.io.*;

/**
 * 来表示的聊天室的一个类. 用于服务器和客户端, 该类包含聊天室和其他属性。在聊天室中
 * 增加了用户列表，即邀请到聊天室的用户。* 
 * @author zcj
 */
public class DushiChatRoom {
	// 这个类是用来管理聊天室.因为它同时可用于服务器和客户端，它定义在单独的文件内。

	private FileOutputStream chatLogStream;
	protected String name = "";
	protected String creatorName = "";
	protected boolean priv;
	protected String password = "";
	protected Vector users;
	protected Vector canvasHistory;
	protected Vector invitedUsers;
	protected Vector bannedUserNames;

	public DushiChatRoom(String n, String c, boolean pr, String pa) {
		name = n;
		creatorName = c;
		priv = pr;
		if (pa != null)
			password = pa;
		users = new Vector();
		canvasHistory = new Vector();
		if (priv)
			invitedUsers = new Vector();
		bannedUserNames = new Vector();
	}

	/**
	 * Add a client to this chatroom
	 */
	protected void addUser(DushiServerConnection newConnection) {
		users.addElement(newConnection);

		// 在用户对象内设置一个引用到该聊天室
		newConnection.user.chatroom = this;
	}

	/**
	 * 从这个聊天室中删除客户端
	 */
	protected void removeUser(DushiServerConnection oldConnection) {
		users.removeElement(oldConnection);
		users.trimToSize();
	}

	/**
	 * 邀请客户到这个聊天室
	 */
	protected void inviteUser(int userId) {
		// 如果这是一个私人房间，我们只保留受邀请的用户的一个向量容器
		if (priv) {
			Integer Id = new Integer(userId);
			invitedUsers.addElement(Id);
		}
	}

	/**
	 * 从邀请的用户列表中删除用户ID
	 */
	protected void uninviteUser(int userId) {
		//如果这是一个私人房间，我们只保留受邀请的用户的一个向量容器
		if (priv) {
			for (int count = 0; count < invitedUsers.size(); count++) {
				Integer Id = new Integer(userId);

				if ((Integer) invitedUsers.elementAt(count) == Id) {
					invitedUsers.removeElement(invitedUsers.elementAt(count));
					invitedUsers.trimToSize();
					break;
				}
			}
		}
	}

	/**
	 * 添加用户名到禁用的用户列表
	 */
	protected void banUserName(String userName) {
		bannedUserNames.addElement(userName);
	}

	/**
	 * 从禁用的用户列表中删除用户名 
	 */
	protected void allowUserName(String userName) {
		for (int count = 0; count < bannedUserNames.size(); count++) {
			if (bannedUserNames.elementAt(count).equals(userName)) {
				bannedUserNames.removeElement(bannedUserNames.elementAt(count));
				bannedUserNames.trimToSize();
				break;
			}
		}
	}

	/**
	 * 为该聊天室打开或关闭日志记录
	 */
	protected void setLogging(boolean onOff) throws IOException {
		if (onOff) {
			// 打开日志文件
			File chatLogFile = new File(name + ".chatlog");
			chatLogStream = new FileOutputStream(chatLogFile);
		} else {
			// 关闭任何现有的日志文件
			if (chatLogStream != null) {
				chatLogStream.close();
				chatLogStream = null;
			}
		}

	}

	protected void log(String logText) throws IOException {
		if (chatLogStream != null) {
			try {
				synchronized (chatLogStream) {
					byte[] bytes = logText.getBytes();
					chatLogStream.write(bytes);
				}
			} catch (IOException e) {
				chatLogStream.close();
				chatLogStream = null;
				throw (e);
			}
		}
	}

	protected void finalize() {
		// 聊天室正在消失。关闭日志文件.
		try {
			chatLogStream.close();
		} catch (IOException e) {
		}
	}
}
